#ifndef __SOVA_H__
#define __SOVA_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <gst/gst.h>
#include <stdio.h>

#include "deepstream_app_version.h"
#include "deepstream_common.h"
#include "deepstream_config.h"
#include "deepstream_config_file_parser.h"
#include "deepstream_perf.h"
#include "deepstream_sinks.h"
#include "deepstream_sources.h"
#include "deepstream_streammux.h"
#include "deepstream_osd.h"

  typedef struct _AppCtx AppCtx;
  typedef struct
  {
    gboolean enable_perf_measurement;
    gboolean source_list_enabled;
    guint total_num_sources;
    guint num_source_sub_bins;
    guint num_sink_sub_bins;
    guint perf_measurement_interval_sec;

    gchar **uri_list;
    NvDsSourceConfig multi_source_config[MAX_SOURCE_BINS];
    NvDsStreammuxConfig streammux_config;
    NvDsOSDConfig osd_config;
    NvDsSinkSubBinConfig sink_bin_sub_bin_config[MAX_SINK_BINS];
  } NvDsConfig;

  typedef struct
  {
    gulong all_bbox_buffer_probe_id;
    GstElement *bin;
    NvDsOSDBin osd_bin;
    NvDsSinkBin sink_bin;
    GstElement *tee;
    GstElement *fake_sink;
    gulong probe_id;
  } NvDsInstanceBin;

  typedef struct
  {
    guint bus_id;
    GstElement *pipeline;
    NvDsSrcParentBin multi_src_bin;
    NvDsStreammuxConfig streammux_config;
    GstElement *demuxer;
    NvDsInstanceBin instance_bins[MAX_SOURCE_BINS];
  } NvDsPipeline;

  struct _AppCtx
  {
    GMutex app_lock;
    GCond app_cond;

    NvDsConfig config;
    NvDsPipeline pipeline;
    NvDsAppPerfStructInt perf_struct;
  };

  /**
   * @brief  Create DS Anyalytics Pipeline per the appCtx
   *         configurations
   * @param  appCtx [IN/OUT] The application context
   *         providing the config info and where the
   *         pipeline resources are maintained
   */
  gboolean create_pipeline(AppCtx *appCtx);

  /**
   * Function to destroy pipeline and release the resources, probes etc.
   */
  void destroy_pipeline(AppCtx *appCtx);

  /**
   * Function to read properties from configuration file.
   *
   * @param[in] config pointer to @ref NvDsConfig
   * @param[in] cfg_file_path path of configuration file.
   *
   * @return true if parsed successfully.
   */
  gboolean
  parse_config_file(NvDsConfig *config, gchar *cfg_file_path);

#ifdef __cplusplus
}
#endif

#endif
