#ifndef __BINS_H__
#define __BINS_H__

#include "../../main.h"

/**
 * @brief Create a source bin.
 *
 * @param pipeline - main pipeline structure
 * @param config - main config of the app
 * @param lastElem - last element which created to correctly link pipeline bins.
 * Here it will be assigned as multi_source_bin.
 * @return gboolean returns TRUE - if everything is OK, else FALSE
 */
gboolean
create_source(NvDsPipeline *pipeline, NvDsConfig *config, GstElement **lastElem);

gboolean create_demuxer(NvDsPipeline *pipeline, NvDsConfig *config, GstElement *lastElem);

/**
 * @brief Function to add components to pipeline which are dependent on number
 * of streams. These components work on single buffer. If tiling is being
 * used then single instance will be created otherwise < N > such instances
 * will be created for < N > streams
 *
 * Creating bin for sinking - output stream (for example in a file)
 *
 * @param instance_bin - bin responsible for stream output. This bin will be created inside this function.
 * @param osd_config - "on screen display" config. Responsible for bounding boxes output.
 * @param sink_bin_sub_bin_config_array - array of sink configs.
 * All sink config parameters you can find in config file in [sink%d] section.
 * @param output_file_path - output file path
 * @param num_sink_sub_bins - number of sinks.
 * @param index - sink index
 * @return gboolean returns TRUE - if everything is OK, else FALSE
 */
gboolean
create_sink_instance_bin(NvDsInstanceBin *instance_bin, NvDsOSDConfig *osd_config,
                         NvDsSinkSubBinConfig *sink_bin_sub_bin_config_array, guint num_sink_sub_bins);

/**
 * @param pipeline - main pipeline structure.
 * @param config - main config of the app
 * @param appCtx - main structure of app that contains all the information about the application.
 * This variable uses latter in process_metadata callback function.
 * @return gboolean returns TRUE - if everything is OK, else FALSE
 */
gboolean create_sink(NvDsPipeline *pipeline, NvDsConfig *config, AppCtx *appCtx);

#endif